'From Pharo5.0 of 16 April 2015 [Latest update: #50658] on 24 March 2016 at 1:50:27.822922 am'!
TestCase subclass: #ConcurrencyTestCase
	instanceVariableNames: 'forkedProcesses log'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Kernel-Tests-Processes'!
Smalltalk renameClassNamed: #NewMutex as: #ExperimentalMutex!
Object subclass: #ExperimentalMutex
	instanceVariableNames: 'lock'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Kernel-Processes'!

!ExperimentalMutex commentStamp: 'BenComan 3/24/2016 00:55' prior: 0!
I represent a light-weight MUTual EXclusion object for when two or more processes need to concurrently access a shared resource. 

Ownership of my lock is granted to a single process and subsequent processes trying to aquire the lock are suspended until the lock is released. Waiting processes are granted access in the order the access was requested.

I make use of the primitives supplied by OwnedLock.

Instance variables:
	lock 	<OwnedLock>		The (primitive) criticalsection/lock.


_Public API and Key Messages_

  #critical -  I ensure the block passed to me can only be executed by one process at a time. 


_Example_

sharedCounter := 0.
mutex := Mutex new
100  timesRepeat: [
	mutex critical: [  
		sharedCounterCopy  :=  sharedCounter.
		Processor yield.
		sharedCounter := sharedCounterCopy + 1.
		Transcript crShow: sharedCounter.
		].
	].
!

ConcurrencyTestCase subclass: #MutexTest
	instanceVariableNames: 'mutex'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Kernel-Tests-Processes'!
MutexTest class
	instanceVariableNames: ''!
LinkedList subclass: #OwnedLock
	instanceVariableNames: 'owner'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Kernel-Processes'!

!OwnedLock commentStamp: 'BenComan 2/21/2016 01:14' prior: 0!
I supply ownership primitives to implement different kind of concurrency tools like Mutex, Monitor, ReadWriteLock and others.
I allow precisely one process acquire me at any one time. I allow recursive acquiring inside same process. 
I was originally implemented as CriticalSection by Eliot Miranda [1] which had high level methods like #critical: and #critical:IfLocked:. But idea around me is to provide only primitives and push high level method on more appropriate object like Mutex and Monitor. 

Public API and Key Messages

-  waitAcquire 
This method acquires a lock for current process. If it is already acquired by another process I will suspend current process. So it will wait until I will be released.
I return true immediatelly if I was already acquired by current process (which means recursive call).   Otherwise I return false (with possible waiting)

- tryAcquire
Same as #acquire but It not suspends current process  for waiting. Instead it return nil immediatelly. So this method ensure no waiting. With #tryAcquire current process will acquires me or not immediatelly.

#waitAcquire and #tryAcquire return true/false with very inconvinent logic. It should be inverted in future which will allow more readable code like: 
	acquiredRightNow := lock acquire.
		or
	releaseRequired  := lock acquire.
But current primitives lead to bad names like: 
	releaseNotRequired  := lock acquire.
	
- release 
This method release me from owner process.
    
	Instance Variables
	owner		<Process>

[1] MIT License - Copyright (c) 2016 - 3D Immersive Collaboration Consulting, LLC.!

Error subclass: #OwnedLockBadRelease
	instanceVariableNames: 'lockOwner perpetrator'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Kernel-Processes'!
ConcurrencyTestCase subclass: #OwnedLockTest
	instanceVariableNames: 'lock'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Kernel-Tests-Processes'!
Error subclass: #ProcessTerminatedInCriticalSection
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Kernel-Processes'!

!ConcurrencyTestCase methodsFor: 'helpers' stamp: 'BenComan 2/21/2016 00:13'!
assertLogEquals: expectedArray
	self assert: self logFlush asArray equals: expectedArray.! !

!ConcurrencyTestCase methodsFor: 'helpers' stamp: 'BenComan 2/17/2016 21:32'!
newHigherProcess: aBlock 
	^self newHigherProcess: aBlock with: {}! !

!ConcurrencyTestCase methodsFor: 'helpers' stamp: 'BenComan 2/20/2016 21:47'!
assertLogEquals: expectedArray forkHigher: aBlock
	self forkHigher: aBlock.
	self assertLogEquals: expectedArray.
! !

!ConcurrencyTestCase methodsFor: 'helpers' stamp: 'BenComan 2/20/2016 03:15'!
newLowerProcess: aBlock 
	^self newLowerProcess: aBlock with: {}! !

!ConcurrencyTestCase methodsFor: 'helpers' stamp: 'BenComan 2/21/2016 00:12'!
logFlush
	| result |
	result := OrderedCollection new.
	log flush: [:item| result add: item].
	^result asArray.! !

!ConcurrencyTestCase methodsFor: 'helpers' stamp: 'BenComan 2/20/2016 19:13'!
makeError
	1/0.! !

!ConcurrencyTestCase methodsFor: 'helpers' stamp: 'BenComan 2/20/2016 18:18'!
log: anObject
	log nextPut: anObject.
! !

!ConcurrencyTestCase methodsFor: 'helpers' stamp: 'BenComan 2/17/2016 21:33'!
waitLastProcessFinished
	| lastProcess |
	lastProcess := forkedProcesses last.
	Processor yield.
	[
		[ lastProcess isTerminated | lastProcess isSuspended] whileFalse: [5 milliSeconds wait ]
	] valueWithin: 2 seconds onTimeout: [ self fail: 'Last process not terminated within 2 seconds' ].

	lastProcess isTerminated ifFalse: [ self notify: 'Last process is not terminated but suspended' ]! !

!ConcurrencyTestCase methodsFor: 'helpers' stamp: 'DenisKudryashov 1/8/2016 16:16'!
fork: aBlock at: priority

	| newProcess |
	newProcess := aBlock forkAt: priority named: testSelector.
	forkedProcesses add: newProcess.
	^newProcess! !

!ConcurrencyTestCase methodsFor: 'helpers' stamp: 'DenisKudryashov 1/8/2016 16:16'!
fork: aBlock

	| newProcess |
	newProcess := aBlock forkNamed: testSelector.
	forkedProcesses add: newProcess.
	^newProcess! !

!ConcurrencyTestCase methodsFor: 'helpers' stamp: 'BenComan 2/20/2016 19:13'!
newProcess: aBlock with: anArray
	| newProcess |
	newProcess := (aBlock newProcessWith: anArray).
	newProcess name: testSelector.
	forkedProcesses add: newProcess.
	^newProcess! !

!ConcurrencyTestCase methodsFor: 'helpers' stamp: 'BenComan 2/17/2016 21:32'!
forkHigher: aBlock 
	^ self fork: aBlock at: Processor activeProcess priority + 1
! !

!ConcurrencyTestCase methodsFor: 'helpers' stamp: 'BenComan 2/20/2016 03:13'!
newProcess: aBlock 
	^self newProcess: aBlock with: {}! !

!ConcurrencyTestCase methodsFor: 'helpers' stamp: 'DenisKudryashov 1/29/2016 14:46'!
forkAndWait: aBlock

	self fork: aBlock.
	self waitLastProcessFinished! !

!ConcurrencyTestCase methodsFor: 'helpers' stamp: 'BenComan 2/20/2016 02:32'!
forkLower: aBlock 
	^ self fork: aBlock at: Processor activeProcess priority - 1
! !

!ConcurrencyTestCase methodsFor: 'helpers' stamp: 'BenComan 2/20/2016 22:12'!
terminateMe
	Processor activeProcess terminate! !

!ConcurrencyTestCase methodsFor: 'helpers' stamp: 'BenComan 3/24/2016 00:26'!
higherThan: aProcess fork: aBlock 
	^ self fork: aBlock at: aProcess priority + 1
! !

!ConcurrencyTestCase methodsFor: 'helpers' stamp: 'BenComan 2/20/2016 19:12'!
newHigherProcess: aBlock with: anArray
	| newProcess |
	newProcess := (aBlock newProcessWith: anArray).
	newProcess priority: Processor activeProcess priority + 1.
	newProcess name: testSelector.
	forkedProcesses add: newProcess.
	^newProcess! !

!ConcurrencyTestCase methodsFor: 'helpers' stamp: 'BenComan 2/20/2016 19:12'!
newLowerProcess: aBlock with: anArray
	| newProcess |
	newProcess := (aBlock newProcessWith: anArray).
	newProcess priority: Processor activeProcess priority - 1.
	newProcess name: testSelector.
	forkedProcesses add: newProcess.
	^newProcess! !

!ConcurrencyTestCase methodsFor: 'running' stamp: 'BenComan 2/20/2016 17:15'!
setUp
	super setUp.
	
	forkedProcesses := OrderedCollection new.
	log := WaitfreeQueue new.! !

!ConcurrencyTestCase methodsFor: 'running' stamp: 'DenisKudryashov 1/8/2016 16:17'!
tearDown
	super tearDown.
	
	forkedProcesses do: #terminate! !


!ConcurrencyTestCase class methodsFor: 'testing' stamp: 'DenisKudryashov 1/30/2016 10:17'!
isAbstract
	^self == ConcurrencyTestCase! !


!Context methodsFor: 'private' stamp: 'BenComan 3/23/2016 22:31'!
"protocol: private"
doPrimitive: primitiveIndex method: meth receiver: aReceiver args: arguments 
	"Simulate a primitive method whose index is primitiveIndex.  The simulated receiver and
	 arguments are given as arguments to this message. If successful, push result and return
	 resuming context, else ^ {errCode, PrimitiveFailToken}. Any primitive which provokes
	 execution needs to be intercepted and simulated to avoid execution running away."

	| value |
	"Judicious use of primitive 19 (a null primitive that doesn't do anything) prevents
	 the debugger from entering various run-away activities such as spawning a new
	 process, etc.  Injudicious use results in the debugger not being able to debug
	 interesting code, such as the debugger itself.  hence use primitive 19 with care :-)"
	"SystemNavigation new browseAllSelect: [:m| m primitive = 19]"
	"primitiveIndex = 19 ifTrue:	[ | debugSession | 
			debugSession := Processor activeProcess newDebugSessionNamed: 'Code simulation error' startedAt: self.
			Smalltalk tools debugger openOn: debugSession withFullView: true]."

	((primitiveIndex between: 201 and: 222)
	 and: [(self objectClass: aReceiver) includesBehavior: BlockClosure]) ifTrue:
		[((primitiveIndex between: 201 and: 205)			 "BlockClosure>>value[:value:...]"
		  or: [primitiveIndex between: 221 and: 222]) ifTrue: "BlockClosure>>valueNoContextSwitch[:]"
			[^aReceiver simulateValueWithArguments: arguments caller: self].
		 primitiveIndex = 206 ifTrue:						"BlockClosure>>valueWithArguments:"
			[^aReceiver simulateValueWithArguments: arguments first caller: self]].

	primitiveIndex = 83 ifTrue: "afr 9/11/1998 19:50" "Object>>perform:[with:...]"
		[^self send: arguments first to: aReceiver with: arguments allButFirst super: false].
	primitiveIndex = 84 ifTrue: "afr 9/11/1998 19:50 & eem 8/18/2009 17:04" "Object>>perform:withArguments:"
		[^self send: arguments first to: aReceiver with: (arguments at: 2) lookupIn: (self objectClass: aReceiver)].
	primitiveIndex = 100 ifTrue: "eem 8/18/2009 16:57" "Object>>perform:withArguments:inSuperclass:"
		[^self send: arguments first to: aReceiver with: (arguments at: 2) lookupIn: (arguments at: 3)].

	"Mutex>>primitiveEnterCriticalSection
	 Mutex>>primitiveTestAndSetOwnershipOfCriticalSection"
	(primitiveIndex = 186 or: [primitiveIndex = 187]) ifTrue:
		[| active effective |
		 active := Processor activeProcess.
		 effective := active effectiveProcess.
		 "active == effective"
		 value := primitiveIndex = 186
					ifTrue: [aReceiver primitiveEnterCriticalSectionOnBehalfOf: effective]
					ifFalse: [aReceiver primitiveTestAndSetOwnershipOfCriticalSectionOnBehalfOf: effective].
		 ^(self isPrimFailToken: value)
			ifTrue: [value]
			ifFalse: [self push: value]].

	primitiveIndex = 188 ifTrue: "eem 5/27/2008 11:10 Object>>withArgs:executeMethod:"
		[arguments size = 2 
			ifTrue: "normal primitive"
				[^Context
					sender: self
					receiver: aReceiver
					method: (arguments at: 2)
					arguments: (arguments at: 1)]
			ifFalse: "mirror primitive"
				[^Context
					sender: self
					receiver: (arguments at: 1)
					method: (arguments at: 3)
					arguments: (arguments at: 2)]
		].
	"Closure primitives"
	(primitiveIndex = 200 and: [self == aReceiver]) ifTrue:
		"ContextPart>>closureCopy:copiedValues:; simulated to get startpc right"
		[^self push: (BlockClosure
						outerContext: aReceiver
						startpc: pc + 2
						numArgs: arguments first
						copiedValues: arguments last)].

	(primitiveIndex between: 207 and: 209) ifTrue: 
		[^OwnedLock simulatePrimitive: primitiveIndex receiver: aReceiver].

	primitiveIndex = 118 ifTrue: "tryPrimitive:withArgs:; avoid recursing in the VM"
		[(arguments size = 2
		 and: [arguments first isInteger
		 and: [(self objectClass: arguments last) == Array]]) ifFalse:
			[^Context primitiveFailTokenFor: nil].
		 ^self doPrimitive: arguments first method: meth receiver: aReceiver args: arguments last].

	value := primitiveIndex = 120 "FFI method"
				ifTrue: [(meth literalAt: 1) tryInvokeWithArguments: arguments]
				ifFalse:
					[primitiveIndex = 117 "named primitives"
						ifTrue: [self tryNamedPrimitiveIn: meth for: aReceiver withArgs: arguments]
						ifFalse: [aReceiver tryPrimitive: primitiveIndex withArgs: arguments]].

	^(self isPrimFailToken: value)
		ifTrue: [value]
		ifFalse: [self push: value]! !


!MutexTest methodsFor: 'tests' stamp: 'DenisKudryashov 1/11/2016 11:52'!
testExecutionCriticalSectionWithLockedHandlerButWhenItIsFree

	| actual |
	
	actual := mutex critical: [ #result ] ifLocked: [].
	
	self assert: actual equals: #result! !

!MutexTest methodsFor: 'tests' stamp: 'BenComan 3/23/2016 23:16'!
testCriticalErrorHandlerForwardsSubsequentErrorsOutside
	"Dennis, I disagree errors shold be handled outside outside the critical section.  
	Errors prevent mutuallyExcluded block from completing, leaving invariants
	in a mess.  
	
	First level handling should be done inside the lock. This handler can raise
	a subsquent error to be handled outside the lock."

	| lastCriticalExecutedBeforeErrorHandler lastCriticalExecuted firstError subsequentError internalLock |
	internalLock := mutex instVarNamed: 'lock'.
	lastCriticalExecutedBeforeErrorHandler := nil.
	lastCriticalExecuted := false.
	firstError := Error new messageText: 'first error'.
	subsequentError := Error new messageText: 'subsequent error'.
	
	self assert: internalLock testSupportStatus equals: #lockFree.
	self 
		assertLogEquals: {	firstError. #lockOwnedByMe. #errorHandledInsideLock. 
								subsequentError. #lockFree. #errorReceivedOutside   }
		forkHigher:  
		[  [ 	mutex 
					critical: [firstError signal]
					ifCurtailedOrError: 
					[	:errorInside |
						self log: errorInside.
						self log: internalLock testSupportStatus.	
						self log: #errorHandledInsideLock. 
						subsequentError signal.
					]
			] on: Error do: 
			[	:errorOutside |
				self log: errorOutside.	
				self log: internalLock testSupportStatus.	 
				self log: #errorReceivedOutside.
			].
		].
	self assert: internalLock testSupportStatus equals: #lockFree.
! !

!MutexTest methodsFor: 'tests' stamp: 'BenComan 2/20/2016 01:47'!
testCriticalWithErrorHandlerShouldCullErrorToHandler

	| actual error |
	error := Error new.
	
	[	mutex critical: [ error signal ] ] on: Error do: [ :err | actual := err ].
	
	self assert: actual equals: error! !

!MutexTest methodsFor: 'tests' stamp: 'DenisKudryashov 1/8/2016 18:28'!
testTerminatingBlockedCriticalSectionShouldNotUnblockAnotherWaitingSection

	| lastExecuted lastExecutedAfterFirst secondCritical |
	lastExecuted := false.
	lastExecutedAfterFirst := nil.
	
	self fork: [mutex critical: [ 10 milliSeconds wait.
			lastExecutedAfterFirst := lastExecuted not ]].
	
	secondCritical := self fork: [mutex critical: [ lastExecuted := true ]].
	self fork: [mutex critical: [ lastExecuted := true ]].
	5 milliSeconds wait.
	secondCritical terminate.
	10 milliSeconds wait.
	
	self assert: lastExecuted.
	self assert: lastExecutedAfterFirst! !

!MutexTest methodsFor: 'tests' stamp: 'BenComan 2/21/2016 18:03'!
testUserErrorHandlingInsideCritical

	| handled |
	handled := false.
	mutex critical: 
	[  	[1/0] on: Error do: [handled := true] 
	].
	
	self assert: handled ! !

!MutexTest methodsFor: 'tests' stamp: 'BenComan 3/23/2016 23:15'!
testCriticalWithCurtailedHandlerShouldExecuteItWhenItWasTerminatedInsideCriticalSection

	| executed process internalLock |
	internalLock := mutex instVarNamed: 'lock'.
	executed := false.
	process := self fork: [mutex critical: [ 10 seconds wait. self inform: internalLock owner  ] ifCurtailedOrError: [executed := true]].
	Processor yield.
	process terminate.
	
	self assert: executed! !

!MutexTest methodsFor: 'tests' stamp: 'DenisKudryashov 1/11/2016 11:51'!
testExecutionCriticalSectionWithLockedHandler

	| actual |
	
	self fork: [ mutex critical: [ 10 seconds wait ] ].
	self fork: [actual := mutex critical: [  ] ifLocked: [#result]].
	self waitLastProcessFinished.
	
	self assert: actual equals: #result! !

!MutexTest methodsFor: 'tests' stamp: 'BenComan 2/20/2016 22:56'!
testCriticalWithCurtailedHandlerShouldNotExecuteItWhenItWasTerminatedBeforeCriticalSectionWasEntered

	| executed process |
	executed := false.

	process := self fork: [ mutex critical: [ 15 milliSeconds wait ] ].	
	self fork: [mutex critical: [ 10 seconds wait ] ifCurtailedOrError: [executed := true]].
	Processor yield.
	process terminate.
	
	self deny: executed! !

!MutexTest methodsFor: 'tests' stamp: 'BenComan 3/23/2016 23:42'!
testSystemSemanticsCurtailVersusErrors
"	Outside of a mutex, certain invariants are assumed, such that data structures 
	are in a 'good' shape.  Only while the lock is held should invariants be violated, 
	and these invariants are expected to be *restored* when the lock is released.  
	However only the mutuallyExcludedBlock knows how to properly restore them. 
	If it doesn't complete due to process termination or an error, data structures can
	be left in a mess.  

	The events that can cause this are...
	  * process termination - handled by #ifCurtailed:
	  * errors - handled by #on:do:
	so we must wrap one handler inside another. However there is a subtle interaction 
	between #ifCurtailed: and #on:do: depending on the order of which wraps which, 
	as presented below. Essentially, 1a and 1b run only their respective handlers, 
	whereas 2a runs both but 2b does not.  So 'error handler inside ifcurtailed' is
	more predictable. This fed into the design of critical:ifCurtailedOrError: 
	
	No waranty is made regarding *why* these cases below behave as the do,
	or if this is desired behaviour. These cases are just to understand the behaviour,
	record current system semantics, and identify if they ever change.	
	"		

	"CASE 1. ERROR HANDLER INSIDE IFCURTAILED"

	"1a. Error in critical section"
	self assertLogEquals: { #criticalSection . #errorHandler }
		forkHigher: 
		[ 	mutex critical: [ 	
				[ 	
					[self log: #criticalSection ; makeError] on: Error do: [self log: #errorHandler] 
				] ifCurtailed: [self log: #curtailedHandler] 
			] 
		].

	"1b. Termination of critical section "
	self assertLogEquals: { #criticalSection . #curtailedHandler }
		forkHigher: 
		[ 	mutex critical: [ 	
				[ 	
					[self log: #criticalSection ; terminateMe ] on: Error do: [self log: #errorHandler] 
				] ifCurtailed: [self log: #curtailedHandler] 
			] 
		].
		
		

	"CASE 2. IFCURTAILED INSIDE ERROR HANDLER"
	
	"2a. Error in critical section"
	self assertLogEquals: { #criticalSection . #errorHandler . #curtailedHandler }
		forkHigher: 
		[ 	mutex critical: [
				[ 	
					[self log: #criticalSection ; makeError] ifCurtailed: [self log: #curtailedHandler] 
				] on: Error do: [self log: #errorHandler] 
			] 
		].
		
	"2b. Termination of critical section "
	self assertLogEquals: { #criticalSection . #curtailedHandler }
		forkHigher: 
		[ 	mutex critical: [ 	
				[ 	
					[self log: #criticalSection ; terminateMe] ifCurtailed: [self log: #curtailedHandler] 
				] on: Error do: [self log: #errorHandler] 
			] 
		].! !

!MutexTest methodsFor: 'tests' stamp: 'BenComan 2/20/2016 02:20'!
testCriticalWithErrorHandlerWhenNoErrorHappens
	"Dennis, not sure if this test is required anymore"
	| actual |
	
	actual := [ mutex critical: [ #result ] ] on: Error do: [  ].
	
	self assert: actual equals: #result! !

!MutexTest methodsFor: 'tests' stamp: 'BenComan 3/24/2016 01:39'!
testErrorsHandledWhileLockHeld
	self forkHigher: [ 
		[ mutex critical: [ self error: 'test' ] ] on: Error do: [ 
			self log: 1. 
			40 milliSeconds wait.
			self log: 2.
			]. 
		].
	self forkHigher: [mutex critical: [ self log: 3.]].
	60 milliSeconds wait.
	
	self assertLogEquals: { 1 . 2 . 3}.! !

!MutexTest methodsFor: 'tests' stamp: 'BenComan 2/20/2016 02:19'!
testCriticalWithErrorHandlerWhenErrorHappens
	"Dennis, not sure this test is required anymore"
	| actual |
	
	actual := [ mutex critical: [ self error: 'test' ]] on: Error do: [ #result ].
	
	self assert: actual equals: #result! !

!MutexTest methodsFor: 'running' stamp: 'BenComan 3/24/2016 00:45'!
setUp
	super setUp.

	mutex := ExperimentalMutex new.
	! !


!OwnedLock methodsFor: 'controlling' stamp: 'BenComan 2/21/2016 13:24'!
waitAcquire
	"Primitive. Blocking attempt to acquire an OwnedLock. 
	 * 	If not owned, set the owner to the current process and answer TRUE, 
		since the lock is acquired here.
	 * If owned by another process, suspend current process such that TRUE
	   is answered when it is resumed by primitiveOwnedLockRelease, 
	   since effectively that is when the lock is aquired by the current process.
	 * If already owned by current process answer NIL, 
	   since lock is reused rather than aquired here.			
	 * 	An argument indicates simulation, which is taken to be the effective activeProcess
	   (see Process>>effectiveProcess)."

	<primitive: 209>
	self primitiveFail
	"In the spirit of the following...
		[	owner ifNil: [owningProcess := Processor activeProcess. ^true].
			owner = Processor activeProcess ifTrue: [^nil].
			self addLast: Processor activeProcess.
	  		Processor activeProcess suspend.			
			^true.
		] valueUnpreemptively
	"
! !

!OwnedLock methodsFor: 'controlling' stamp: 'BenComan 3/23/2016 23:01'!
release
	"Release of lock by the *owner*  
	 Fail if a non-owner tries to release the lock. Non-owners must use #break.
	"
		
	self primitiveOwnedLockReleaseOnBehalfOf: Processor activeProcess.


	"BenComan 2016.02.21 There is a strange scenario here when running test
			MutexTest>>testCriticalWithCurtailedHandlerShouldExecuteItWhenItWasTerminatedInsideCriticalSection
	It works when the image passes the active process to the primitive, if the line above is...
	    self primitiveOwnedLockReleaseOnBehalfOf: Processor activeProcess.
	but fails when the primitive determines that active process, if the line above is... 
		 self releasePrimitive.   
	The failure comes from desired behaviour in VM>>primitiveOwnedLockRelease...
	    (owningProcess = activeProcess) ifFalse: [self primitiveFail].
   except activeProcess seems incorrect when the process it terminating (or my understanding in lacking)

	For now it works fine passing the active process from the image. The primitive 
	" 
! !

!OwnedLock methodsFor: 'controlling' stamp: 'BenComan 2/21/2016 13:23'!
tryAcquire
	"Primitive. Non-blocking attempt to acquire an OwnedLock. 
	 * 	If not owned, set the owner to the current process and answer TRUE, 
		since the lock is acquired here.
	 * If owned by another process answer FALSE, 
	   since the lock could not be aquired.
	 * If already owned by current process, answer NIL, 
	   since lock is reused rather than aquired here.	
	 * 	An argument indicates simulation, which is taken to be the effective activeProcess
	   (see Process>>effectiveProcess)."

	<primitive: 208>
	self primitiveFail
	"In the spirit of the following...
		[	owner ifNil: [owningProcess := Processor activeProcess. ^true].
			owner = Processor activeProcess ifTrue: [^nil].
			^false
		] valueUnpreemptively
	"! !

!OwnedLock methodsFor: 'controlling' stamp: 'BenComan 2/21/2016 17:09'!
break
	"Only the owner of a lock *should* be able to release it.  
	 If anyone else does it, the lock is considered to be broken."
	self primitiveOwnedLockReleaseOnBehalfOf: owner! !

!OwnedLock methodsFor: 'printing' stamp: 'BenComan 2/21/2016 01:07'!
printElementsOn: aStream
	aStream nextPut: $(.
	owner 
		ifNil: [aStream nextPutAll: 'FREE'] 
		ifNotNil: [owner hash printOn: aStream. aStream space. owner printOn: aStream].
	aStream nextPut: $)! !

!OwnedLock methodsFor: 'testing' stamp: 'BenComan 2/20/2016 23:25'!
isFree
	^ owner = nil! !

!OwnedLock methodsFor: 'testing' stamp: 'BenComan 2/20/2016 23:25'!
isOwnedByMe
	^ owner = Processor activeProcess! !

!OwnedLock methodsFor: '*Kernel-Tests' stamp: 'BenComan 2/21/2016 01:17'!
testSupportStatus
	"Purely for use by logging in the unit tests."
	owner = nil ifTrue: [ ^ #lockFree ].
	owner = Processor activeProcess
		ifTrue: [ ^ #lockOwnedByMe ]
		ifFalse: [ ^ #lockOwnedBySomeoneElse ].
! !

!OwnedLock methodsFor: 'private' stamp: 'BenComan 2/21/2016 20:42'!
releasePrimitive
	"Primitive. Release of lock by the *owner*  
	 * Fail if a non-owner tries to release the lock. Non-owners must use #break.
	 * 	An argument indicates simulation, which is taken to be the effective activeProcess
	   This mechanism can be used by a non-owner to force/break the lock. 
	 * Active process may be changed as a result.
	 * Based on primitiveExitCriticalSection, but modified to be strict.
		For a non-strict release, the image can use primitiveExitCriticalSection directly "
	<primitive: 207>
	| error |
	error := OwnedLockBadRelease lockOwner: owner perpetrator: Processor activeProcess.
	error signal.
! !

!OwnedLock methodsFor: 'simulating' stamp: 'BenComan 2/21/2016 16:52'!
primitiveOwnedLockTryAcquireOnBehalfOf: effectiveProcess
	"Primitive. Non-blocking attempt to acquire an OwnedLock. 
	 * 	If not owned, set the owner to the current process and answer TRUE, 
		since the lock is acquired here.
	 * If owned by another process answer FALSE, 
	   since the lock could not be aquired.
	 * If already owned by current process, answer NIL, 
	   since lock is reused rather than aquired here.	
	 * 	An argument indicates simulation, which is taken to be the effective activeProcess
	   (see Process>>effectiveProcess)."
	<primitive: 208>
	self primitiveFailed
	"In the spirit of the following"
	"[owner ifNil:
		[owner := Processor activeProcess.
		 ^false].
	  owner = Processor activeProcess ifTrue:
		[^true].
	  self addLast: Processor activeProcess.
	  Processor activeProcess suspend] valueUnpreemptively"! !

!OwnedLock methodsFor: 'simulating' stamp: 'BenComan 2/21/2016 16:52'!
primitiveOwnedLockWaitAcquireOnBehalfOf: effectiveProcess
	"Primitive. Blocking attempt to acquire an OwnedLock. 
	 * 	If not owned, set the owner to the current process and answer TRUE, 
		since the lock is acquired here.
	 * If owned by another process, suspend current process such that TRUE
	   is answered when it is resumed by primitiveOwnedLockRelease, 
	   since effectively that is when the lock is aquired by the current process.
	 * If already owned by current process answer NIL, 
	   since lock is reused rather than aquired here.			
	 * 	An argument indicates simulation, which is taken to be the effective activeProcess
	   (see Process>>effectiveProcess)."

	<primitive: 209>
	self primitiveFail
	"In the spirit of the following...
		[	owner ifNil: [owningProcess := Processor activeProcess. ^true].
			owner = Processor activeProcess ifTrue: [^nil].
			self addLast: Processor activeProcess.
	  		Processor activeProcess suspend.			
			^true.
		] valueUnpreemptively
	"
! !

!OwnedLock methodsFor: 'simulating' stamp: 'BenComan 2/21/2016 16:52'!
primitiveOwnedLockReleaseOnBehalfOf: effectiveProcess
	"Primitive. Non-blocking attempt to acquire an OwnedLock. 
	 * 	If not owned, set the owner to the current process and answer TRUE, 
		since the lock is acquired here.
	 * If owned by another process answer FALSE, 
	   since the lock could not be aquired.
	 * If already owned by current process, answer NIL, 
	   since lock is reused rather than aquired here.	
	 * 	An argument indicates simulation, which is taken to be the effective activeProcess
	   (see Process>>effectiveProcess)."
	<primitive: 207>
	| error |
	error := OwnedLockBadRelease lockOwner: owner perpetrator: Processor activeProcess.
	error signal.! !

!OwnedLock methodsFor: 'accessing' stamp: 'BenComan 2/20/2016 02:03'!
owner
	"For debugging. Answer the owning process of this mutex.
	There is no setter since the owning process is managed by the VM."
	^owner! !


!OwnedLock class methodsFor: 'simulating' stamp: 'BenComan 2/21/2016 16:59'!
simulatePrimitive: primitiveIndex receiver: aReceiver
	| active effective returnValue|
	active := Processor activeProcess.
	effective := active effectiveProcess.
	"active == effective"
	primitiveIndex = 207 ifTrue: [returnValue := aReceiver primitiveOwnedLockReleaseOnBehalfOf:  effective].
	primitiveIndex = 208 ifTrue: [returnValue := aReceiver primitiveOwnedLockTryAcquireOnBehalfOf:  effective].
	primitiveIndex = 209 ifTrue: [returnValue := aReceiver primitiveOwnedLockWaitAcquireOnBehalfOf: effective].
	^(self isPrimFailToken: returnValue)
			ifTrue: [returnValue]
			ifFalse: [self push: returnValue]! !


!OwnedLockBadRelease methodsFor: 'as yet unclassified' stamp: 'BenComan 2/21/2016 15:32'!
defaultAction
	self messageText: 'Inappropriate attempt to release lock'.
	^super defaultAction! !

!OwnedLockBadRelease methodsFor: 'accessing' stamp: 'BenComan 2/21/2016 16:20'!
lockOwner
	^ lockOwner! !

!OwnedLockBadRelease methodsFor: 'accessing' stamp: 'BenComan 2/21/2016 15:47'!
perpetrator
	^ perpetrator! !

!OwnedLockBadRelease methodsFor: 'accessing' stamp: 'BenComan 2/21/2016 20:13'!
gtInspectorItemsIn: composite
	<gtInspectorPresentationOrder: 0>  
	^ (composite table)
		title: 'Error';
		display: [ 
			{	'lockOwner' -> lockOwner. 
				'perpetrator' -> perpetrator
			} ];
		column: 'Item' evaluated: #key;
		column: 'Value' evaluated: #value;
		send: #value.
		! !

!OwnedLockBadRelease methodsFor: 'accessing' stamp: 'BenComan 2/21/2016 20:59'!
printOn: aStream
	super printOn: aStream.
	aStream nextPutAll: '(lockOwner='.
	lockOwner printOn: aStream.
	aStream nextPutAll: ', perpetrator='.
	perpetrator printOn: aStream.
	aStream nextPutAll: ')'.! !

!OwnedLockBadRelease methodsFor: 'accessing' stamp: 'BenComan 2/21/2016 20:56'!
lockOwner: owningProcess perpetrator: perpProcess
	lockOwner := owningProcess.
	perpetrator := perpProcess.
	Transcript crShow: self printString.! !


!OwnedLockBadRelease class methodsFor: 'instance creation' stamp: 'BenComan 2/21/2016 16:24'!
lockOwner: owningProcess perpetrator: perpProcess
	^self new lockOwner: owningProcess perpetrator: perpProcess! !


!OwnedLockTest methodsFor: 'tests' stamp: 'BenComan 2/21/2016 17:12'!
testBreakByNonOwner
	"Only the process that owns the lock should be able to release it"
	| errorRaised brokenLockCouldBeAcquired |
	errorRaised := false.
	brokenLockCouldBeAcquired := false.
	self forkHigher: [ lock waitAcquire ].
	self forkHigher: 
	[ 	[lock break] on: OwnedLockBadRelease do: [ errorRaised := true ].
		lock waitAcquire.
		brokenLockCouldBeAcquired := true.
	].	
	self waitLastProcessFinished.
	self assert: brokenLockCouldBeAcquired.! !

!OwnedLockTest methodsFor: 'tests' stamp: 'BenComan 2/21/2016 17:07'!
testReleaseByNonOwner
	"Only the process that owns the lock should be able to release it"
	| errorRaised |
	errorRaised := false.
	self forkHigher: [ lock waitAcquire ].
	self forkHigher: [ [lock release] on: OwnedLockBadRelease do: [ errorRaised := true ] ].	
	self waitLastProcessFinished.
	self assert: errorRaised! !

!OwnedLockTest methodsFor: 'tests' stamp: 'BenComan 2/21/2016 12:47'!
testAttemptToAcquireLockTwiceFromSameProcess
	"maybe rename testTryAquireFreshLockTwiceFromSameProcess"
	| acquired |
	self fork: [ lock tryAcquire. acquired := lock tryAcquire ].
	self waitLastProcessFinished.
	
	self assert: acquired = nil description: 'reusing acquired lock should return nil'
	! !

!OwnedLockTest methodsFor: 'tests' stamp: 'BenComan 2/21/2016 12:44'!
testAttemptToAcquireBusyLock
	"maybe rename testTryAcquireTwiceFromDifferentProcesses"
	| blocked acquired |
	blocked := true.
	self forkHigher: [ lock tryAcquire ].
	self forkHigher: [ acquired := lock tryAcquire. blocked := false].
	self waitLastProcessFinished.
	
	self deny: acquired description: 'tryAcquire should answer false when lock is busy'. 
	self deny: blocked description: 'tryAcquire should be non-blocking'.
	! !

!OwnedLockTest methodsFor: 'tests' stamp: 'BenComan 2/21/2016 12:41'!
testAcquireLockTwiceFromSameProcess
	"maybe rename testWaitAcquireTwiceFromSameProcess"
	| acquired |
	self forkHigher: [ lock waitAcquire. acquired := lock waitAcquire ].
	self waitLastProcessFinished.
	
	self assert: acquired = nil description: 'reusing acquired lock should return nil'
	! !

!OwnedLockTest methodsFor: 'tests' stamp: 'BenComan 2/21/2016 12:46'!
testAttemptToAcquireFreeLock
	"maybe rename testTryAquireFreshLock"
	| acquired |
	self forkHigher: [ acquired := lock tryAcquire ].
	self waitLastProcessFinished.
	
	self assert: acquired description: 'acquiring free lock should return true'
	! !

!OwnedLockTest methodsFor: 'tests' stamp: 'BenComan 2/21/2016 12:44'!
testAcquireLockWhenItsAlreadyBusy
	"maybe rename testWaitAcquireTwiceFromDifferentProcesses"
	| blocked |
	blocked := true.
	self forkHigher: [ lock waitAcquire ].
	self forkHigher: [ lock waitAcquire. blocked := false ].	
	self assert: blocked description: 'waitAquire should block when lock held by someone else'
	! !

!OwnedLockTest methodsFor: 'tests' stamp: 'BenComan 2/21/2016 17:07'!
testReleaseByOwner
	"Strictly, only the process that owns the lock should be able to release it"
	| errorRaised |
	errorRaised := false.
	self forkHigher: 
		[ 	lock waitAcquire.
			[lock release] on: OwnedLockBadRelease do: [ errorRaised := true ] 
		].	
	self waitLastProcessFinished.
	self deny: errorRaised! !

!OwnedLockTest methodsFor: 'tests' stamp: 'BenComan 2/21/2016 12:41'!
testAcquireFreeLockWhichWasAlreadyInUse
	"maybe rename to testWaitAcquirePreusedLock"
	| acquired |
	self forkHigher: [ lock waitAcquire. lock release ].
	self waitLastProcessFinished.

	self forkHigher: [ acquired := lock waitAcquire ].
	self waitLastProcessFinished.
		
	self assert: acquired description: 'acquiring released lock should return true'
	! !

!OwnedLockTest methodsFor: 'tests' stamp: 'BenComan 2/21/2016 12:41'!
testAcquireFreeLock
	"maybe rename testWaitAcquireFreshLock"
	| acquired |
	self forkHigher: [ acquired := lock waitAcquire ].
	self waitLastProcessFinished.
	
	self assert: acquired description: 'acquiring free lock should return true'

	! !

!OwnedLockTest methodsFor: 'running' stamp: 'DenisKudryashov 1/29/2016 13:20'!
setUp
	super setUp.
	
	lock := OwnedLock new.! !


!Process methodsFor: 'printing' stamp: 'BenComan 2/21/2016 20:26'!
printOn: aStream

	super printOn: aStream.
	aStream nextPutAll: '(' , self hash printString , ') in '.
	suspendedContext printOn: aStream! !


!SmalltalkImage methodsFor: 'sources, change log' stamp: 'DenisKudryashov 1/29/2016 13:41'!
licenseStringTemplate
	^ 'LICENSE
	
Licensed under the MIT License with parts under the Apache License.

Copyright (c) 2008-{1} The Pharo Project, and Contributors
Copyright (c) 2008-{1} Inria
Copyright (c) 1996-2008 Viewpoints Research Institute, and Contributors 
Copyright (c) 1996 Apple Computer, Inc.

Permission is hereby granted, free of charge, to any person obtaining a copy of this
software and associated documentation files (the "Software"), to deal in the Software
without restriction, including without limitation the rights to use, copy, modify, merge,
publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons
to whom the Software is furnished to do so, subject to the following conditions: The
above copyright notice and this permission notice shall be included in all copies or
substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS
OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

You may obtain a copy of the Apache License at
http://www.apache.org/licenses/LICENSE-2.0

Pharo uses icons by Mark James (http://www.famfamfam.com) under 
Creative Commons Attribution 2.5 License.

Pharo uses icons by Eclipse under 
Eclipse Public License - v 1.0
http://www.eclipse.org/legal/epl-v10.html

Pharo uses Source Code Pro Fonts by Adobe under 
SIL Open Font License, v 1.1
http://scripts.sil.org/OFL

Pharo uses Open Sans by Steve Matteson under 
Apache License, 2.0
http://www.apache.org/licenses/LICENSE-2.0.html

Pharo uses  Bitstream Vera Fonts
http://dejavu-fonts.org/wiki/index.php?title=License

Copyright (c) 2003 by Bitstream, Inc. All Rights Reserved. 
Bitstream Vera is a trademark of Bitstream, Inc.
Permission is hereby granted, free of charge, to any person obtaining a copy of the fonts 
accompanying this license ("Fonts") and associated documentation files (the "Font Software"), 
to reproduce and distribute the Font Software, including without limitation the rights to use, copy,
merge, publish, distribute, and/or sell copies of the Font Software, and to permit persons to whom 
the Font Software is furnished to do so, subject to the following conditions:
The above copyright and trademark notices and this permission notice shall be included in all 
copies of one or more of the Font Software typefaces.
The Font Software may be modified, altered, or added to, and in particular the designs of glyphs 
or characters in the Fonts may be modified and additional glyphs or characters may be added to
the Fonts, only if the fonts are renamed to names not containing either the words "Bitstream" 
or the word "Vera".
This License becomes null and void to the extent applicable to Fonts or Font Software that 
has been modified and is distributed under the "Bitstream Vera" names.
The Font Software may be sold as part of a larger software package but no copy of one or more
of the Font Software typefaces may be sold by itself.
THE FONT SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, 
INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR 
PURPOSE AND NONINFRINGEMENT OF COPYRIGHT, PATENT, TRADEMARK, OR OTHER RIGHT. IN NO EVENT 
SHALL BITSTREAM OR THE GNOME FOUNDATION BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, 
INCLUDING ANY GENERAL, SPECIAL, INDIRECT, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, WHETHER IN AN 
ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF THE USE OR INABILITY TO USE THE 
FONT SOFTWARE OR FROM OTHER DEALINGS IN THE FONT SOFTWARE.
Except as contained in this notice, the names of Gnome, the Gnome Foundation, 
and Bitstream Inc., shall not be used in advertising or otherwise to promote the sale,
use or other dealings in this Font Software without prior written authorization from 
the Gnome Foundation or Bitstream Inc., respectively. 
For further information, contact: fonts at gnome dot org.

Pharo uses mutex VM primitives in class OwnedLock under MIT.
Copyright (c) 2016 - 3D Immersive Collaboration Consulting, LLC'! !

MutexTest removeSelector: #fork:!
MutexTest removeSelector: #fork:at:!

!MutexTest reorganize!
(#tests testExecutionCriticalSectionWithLockedHandlerButWhenItIsFree testCriticalErrorHandlerForwardsSubsequentErrorsOutside testTwoCriticalsShouldWaitEachOther testCriticalWithErrorHandlerShouldCullErrorToHandler testTerminatingBlockedCriticalSectionShouldNotUnblockAnotherWaitingSection testUserErrorHandlingInsideCritical testCriticalWithCurtailedHandlerShouldExecuteItWhenItWasTerminatedInsideCriticalSection testExecutionCriticalSectionWithLockedHandler testFailedCriticalSectionShouldUnblockWaitingOne testCriticalWithCurtailedHandlerShouldNotExecuteItWhenItWasTerminatedBeforeCriticalSectionWasEntered testSystemSemanticsCurtailVersusErrors testCriticalWithErrorHandlerWhenNoErrorHappens testWaitingProcessReceivingLockTerminatedBeforeResuming testErrorsHandledWhileLockHeld testCriticalWithErrorHandlerWhenErrorHappens testTerminatedCriticalSectionShouldUnblockWaitingOne testExecutionCriticalSection testTwoRecursiveCriticalsShouldNotWaitEachOther)
(#running setUp tearDown)
!

ExperimentalMutex removeSelector: #testSupportSetLock:!

!ExperimentalMutex reorganize!
(#initialization initialize)
(#'mutual exclusion' critical:ifLocked: critical:ifCurtailedOrError: critical:)
!

